package com.zulutrade.sample.trading.client;
import com.zulutrade.sample.trading.client.configuration.ZuluTradeAuthenticationInfo;
import com.zulutrade.sample.trading.client.configuration.ZuluTradingServerConfig;
import com.zulutrade.sample.trading.client.trading.UniqueIdGenerator;
import com.zulutrade.sample.trading.client.trading.ZuluTradingGateway;
import com.zulutrade.sample.trading.client.trading.ZuluTradingGatewayImpl;
import com.zulutrade.sample.trading.client.trading.dtos.ClosedTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewMarketTradeResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.NewOrderResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OpenTradesResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.OrderCanceledResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.ProviderStatisticsResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradeClosedResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.TradingConfigurationResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateTrailingStopValueResponseMessage;
import com.zulutrade.sample.trading.client.trading.dtos.UpdateValueResponseMessage;
import com.zulutrade.sample.trading.client.utils.DateUtil;


/**
 * Demo application showing how to use ZuluTrading api
 *
 * @author ZuluTrade
 */
public class ZuluTradingSampleApp {



	public static void main( String[] args ){


		ZuluTradingServerConfig cfg = new ZuluTradingServerConfig("http://tradingserver-beta.zulutrade.com/");
		
		ZuluTradeAuthenticationInfo ai = new ZuluTradeAuthenticationInfo("yourZuluAccountUsername", "yourZuluAccountPassword");
		
		ZuluTradingGateway tradingGateway = new ZuluTradingGatewayImpl(ai, cfg);
		/*
		 * Getting minimum trading amount and amount step for account 
		 */	
		try{
			TradingConfigurationResponseMessage tradingConfiguration = tradingGateway.getTradingConfiguration();
			
			System.out.println("Account's trading configuration: ");
			System.out.print("Minimum amount for trading: " +tradingConfiguration.getMinAmount()+ " lots");
			System.out.print(", Minimum amount step for trading: " +tradingConfiguration.getMinAmountStep()+ " lots");
			System.out.println("\n\n");
		}catch(Exception ex){
			ex.printStackTrace();
		}
		
		/*
		 * Get open trades/orders
		 */	
		try{
			OpenTradesResponseMessage openTrades = tradingGateway.getOpenTrades();
			
			System.out.println("Open market trades: ");
			System.out.println("Total: "+openTrades.getOpenPositions().length);
			for(NewMarketTradeResponseMessage marketTrade : openTrades.getOpenPositions()){
				System.out.print("provider ticket: "+marketTrade.getUniqueId());
				System.out.print(" broker ticket: "+marketTrade.getBrokerTicket());
				System.out.print(", currency: "+marketTrade.getCurrencyName());
				System.out.print(", lots: "+marketTrade.getLots());
				System.out.print(", rate: "+marketTrade.getEntryRate());
				System.out.print(", stop: "+marketTrade.getStopValue());
				System.out.print(", trailing stop: "+marketTrade.getTrailingStopValue());
				System.out.print(", trailing stop condition pips: "+marketTrade.getTrailingStopConditionPipsValue());
				System.out.print(", limit: "+marketTrade.getLimitValue());
				System.out.print("\n, floatingPnL: "+marketTrade.getFloatingPnl());
				System.out.println();
			}

			System.out.println("\nPending orders: ");
			System.out.println("Total: "+openTrades.getOpenOrders().length);
			for(NewOrderResponseMessage pendingOrder : openTrades.getOpenOrders()){
				System.out.print("provider ticket: "+pendingOrder.getUniqueId());
				System.out.print(" broker ticket: "+pendingOrder.getBrokerTicket());
				System.out.print(", currency: "+pendingOrder.getCurrencyName());
				System.out.print(", lots: "+pendingOrder.getLots());
				System.out.print(", rate: "+pendingOrder.getEntryRate());
				System.out.print(", stop: "+pendingOrder.getStopValue());
				System.out.print(", trailing stop:"+pendingOrder.getTrailingStopValue());
				System.out.print(", trailing stop condition pips: "+pendingOrder.getTrailingStopConditionPipsValue());
				System.out.print(", limit: "+pendingOrder.getLimitValue());
				System.out.println();
			}
			
		}catch(Exception ex){
			ex.printStackTrace();
		}

		/*
		 * Opening a market order
		 * and setting stop loss, trailing stop, trailing stop condition, take profit 
		 * and closing example
		 */		
		try{
			
			String uniqueId = UniqueIdGenerator.getNextUniqueId();
			
			System.out.println("\n\nOpening trade, uniqueId to use: "+uniqueId);
			
			double entryRateToUse = 1.4;
			NewMarketTradeResponseMessage openResponse = tradingGateway.openMarket("EUR/USD", 1, true, entryRateToUse, uniqueId);    		
			
			if(openResponse.isSuccess()){
				
				System.out.println("\tTrade opened, unique id: "+openResponse.getUniqueId()
						+", brokerTicket: "+openResponse.getBrokerTicket()+", entry rate: "+openResponse.getEntryRate()+", utc date opened: "+openResponse.getDateOpenedAsString()+", timestamp: "+openResponse.getOpenUtcTimestamp());
				
				
				double stopLossToSet = 1.0;
				
				System.out.println("Setting stop loss to "+stopLossToSet);
				
				UpdateValueResponseMessage updateResponse = tradingGateway.updateStop("EUR/USD", true, 1, uniqueId, stopLossToSet);
				
				if(updateResponse.isSuccess()){
					System.out.println("\tSuccess: new "+updateResponse.getValueType()+" value "+updateResponse.getNewValue());
				}else{
					System.err.println("\tFailure: could not set stop loss, error message: "+updateResponse.getErrorMessage()+", error code: "+updateResponse.getErrorCode());
				}

				
				int trailingStopValueToSet = 600;
				
				System.out.println("Setting trailing stop value to "+trailingStopValueToSet);
				
				UpdateTrailingStopValueResponseMessage updateTrailingStopResponse = tradingGateway.updateTrailingStop("EUR/USD", true, 1, uniqueId, trailingStopValueToSet);
				if(updateTrailingStopResponse.getSuccess()){
					System.out.println("\tSuccess: new "+updateTrailingStopResponse.getValueType()+" value set at "+updateTrailingStopResponse.getNewTrailingStopValue()+" pips. New stop value: " +updateTrailingStopResponse.getNewStopValue());
				}else{
					System.err.println("\tFailure: could not set trailing stop, error message: "+updateTrailingStopResponse.getErrorMessage()+", error code: "+updateTrailingStopResponse.getErrorCode());
				}
				
				trailingStopValueToSet = 700;
				
				int tralingStopConditionPipsToSet = 800;
				
				System.out.println("Setting trailing stop conditions value to "+tralingStopConditionPipsToSet+" and trailing stop value to "+trailingStopValueToSet);
				
				updateTrailingStopResponse = tradingGateway.updateTrailingStopCondition("EUR/USD", true, 1, uniqueId, trailingStopValueToSet, tralingStopConditionPipsToSet);
				if(updateTrailingStopResponse.getSuccess()){
					System.out.println("\tSuccess: new "+updateTrailingStopResponse.getValueType()+" value set at "+updateTrailingStopResponse.getNewTrailingStopConditionValue()+" pips. Trailing Stop value that will be set when "+updateTrailingStopResponse.getValueType()+" is reached is: "+updateTrailingStopResponse.getNewTrailingStopValue());
				}else{
					System.err.println("\tFailure: could not set trailing stop condition pips, error message: "+updateTrailingStopResponse.getErrorMessage()+", error code: "+updateTrailingStopResponse.getErrorCode());
				}

				double takeProfitToSet = 2.1;
				
				System.out.println("Setting take profit to "+takeProfitToSet);
				
				updateResponse = tradingGateway.updateLimit("EUR/USD", true, 1, uniqueId, takeProfitToSet);				
				
				if(updateResponse.isSuccess()){
					System.out.println("\tSuccess: new "+updateResponse.getValueType()+" value "+updateResponse.getNewValue());
				}else{
					System.err.println("\tFailure: could not set take profit, error message: "+updateResponse.getErrorMessage()+", error code: "+updateResponse.getErrorCode());
				}
				
				System.out.println("Closing trade");
				
				TradeClosedResponseMessage closedResponse = tradingGateway.closeMarket("EUR/USD", 1, true, uniqueId, entryRateToUse);
				
				if(closedResponse.isSuccess()){
				
					System.out.print("\tTrade closed"+", ");
					System.out.print("price open: "+closedResponse.getPriceOpen()+", ");
					System.out.print("price closed: "+closedResponse.getPriceClosed()+", ");
					System.out.print("date open: "+closedResponse.getDateOpen()+", ");
					System.out.print("open timestamp: "+closedResponse.getOpenUtcTimestamp()+", ");					
					System.out.print("date closed: "+closedResponse.getDateClosed()+", ");
					System.out.print("close timestamp: "+closedResponse.getCloseUtcTimestamp()+", ");
					System.out.print("gross pnl: "+closedResponse.getGrossPnl()+", ");
					System.out.print("interest: "+closedResponse.getInterest()+", ");
					System.out.print("commission :"+closedResponse.getCommission()+", ");
					System.out.println("net pnl: "+closedResponse.getNetPnl());
					
				}else{
					System.err.println("\tCould not close trade, error message: "+closedResponse.getErrorMessage()+", error code: "+updateResponse.getErrorCode());
				}

			}else{
				System.err.println("\tCould not open trade, response: "+openResponse.getErrorMessage()+", error code: "+openResponse.getErrorCode());
			}
		}catch(Exception ex){
			ex.printStackTrace();
		}
	
		
		
		/*
		 * Opening a pending order
		 * and setting stop loss, trailing stop, trailing stop condition, take profit, entry rate 
		 * and canceling example
		 */
		try{
			
			String uniqueId = UniqueIdGenerator.getNextUniqueId();
			
			double entryRateToUse = 1.4;
			System.out.println("\n\nOpenning pending order, uniqueId to use: "+uniqueId+", target entry rate: "+entryRateToUse);
			
			NewOrderResponseMessage openResponse = tradingGateway.openOrder("EUR/USD", 1, true, entryRateToUse, uniqueId);    		
			
			if(openResponse.isSuccess()){
				
				System.out.println("\tPending order placed, unique id: "+openResponse.getUniqueId()
						+", brokerTicket: "+openResponse.getBrokerTicket()+", entry rate: "+openResponse.getEntryRate()+", utc date opened: "+openResponse.getDateOpenedAsString()+" , timestamp: "+openResponse.getOpenUtcTimeStamp());
				
				
				double stopLossToSet = 0.9;
				
				System.out.println("Setting stop loss to "+stopLossToSet);
				
				UpdateValueResponseMessage updateResponse = tradingGateway.updateStop("EUR/USD", true, 1, uniqueId, stopLossToSet);
				
				if(updateResponse.isSuccess()){
					System.out.println("\tSuccess: new "+updateResponse.getValueType()+" value "+updateResponse.getNewValue());
				}else{
					System.err.println("\tFailure: could not set stop loss, error message: "+updateResponse.getErrorMessage());
				}
				
				int trailingStopValueToSet = 600;
				
				System.out.println("Setting trailing stop value to "+trailingStopValueToSet);
				
				UpdateTrailingStopValueResponseMessage updateTrailingStopResponse = tradingGateway.updateTrailingStop("EUR/USD", true, 1, uniqueId, trailingStopValueToSet);
				if(updateTrailingStopResponse.getSuccess()){
					System.out.println("\tSuccess: new "+updateTrailingStopResponse.getValueType()+" value set at "+updateTrailingStopResponse.getNewTrailingStopValue()+" pips. New stop value: " +updateTrailingStopResponse.getNewStopValue());
				}else{
					System.err.println("\tFailure: could not set trailing stop, error message: "+updateTrailingStopResponse.getErrorMessage()+", error code: "+updateTrailingStopResponse.getErrorCode());
				}
				
				trailingStopValueToSet = 700;
				
				int tralingStopConditionPipsToSet = 800;
				
				System.out.println("Setting trailing stop conditions value to "+tralingStopConditionPipsToSet+" and trailing stop value to "+trailingStopValueToSet);

				updateTrailingStopResponse = tradingGateway.updateTrailingStopCondition("EUR/USD", true, 1, uniqueId, trailingStopValueToSet, tralingStopConditionPipsToSet);
				if(updateTrailingStopResponse.getSuccess()){
					System.out.println("\tSuccess: new "+updateTrailingStopResponse.getValueType()+" value set at "+updateTrailingStopResponse.getNewTrailingStopConditionValue()+" pips. Trailing Stop value that will be set when "+updateTrailingStopResponse.getValueType()+" is reached is: "+updateTrailingStopResponse.getNewTrailingStopValue());
				}else{
					System.err.println("\tFailure: could not set trailing stop condition pips, error message: "+updateTrailingStopResponse.getErrorMessage()+", error code: "+updateTrailingStopResponse.getErrorCode());
				}

				double takeProfitToSet = 2.1;
				
				System.out.println("Setting take profit to "+takeProfitToSet);
				
				updateResponse = tradingGateway.updateLimit("EUR/USD", true, 1, uniqueId, takeProfitToSet);				
				
				if(updateResponse.isSuccess()){
					System.out.println("\tSuccess: new "+updateResponse.getValueType()+" value "+updateResponse.getNewValue());
				}else{
					System.err.println("\tFailure: could not set take profit, error message: "+updateResponse.getErrorMessage());
				}
				
				double entryRateToUpdate = 1.5;
				
				System.out.println("Updating entry rate to "+entryRateToUpdate);
				
				updateResponse = tradingGateway.updateEntryRate("EUR/USD", true, 1, uniqueId, entryRateToUpdate);				
				
				if(updateResponse.isSuccess()){
					System.out.println("\tSuccess: new "+updateResponse.getValueType()+" value "+updateResponse.getNewValue());
				}else{
					System.err.println("\tFailure: could not update entry rate, error message: "+updateResponse.getErrorMessage());
				}
				
				System.out.println("Canceling pending order");
				
				OrderCanceledResponseMessage cancelResponse = tradingGateway.cancelOrder("EUR/USD", 1, true, uniqueId);
				
				if(cancelResponse.isSuccess()){				
					System.out.print("\tPending order canceled"+", ");
					System.out.print("date canceled: "+cancelResponse.getDateCanceled()+", timestamp: "+cancelResponse.getCanceledUtcTimestamp());					
				}else{
					System.err.println("\tCould not cancel pending order, error message: "+cancelResponse.getErrorMessage());
				}

			}else{
				System.err.println("\tCould not open pending order, response: "+openResponse.getErrorMessage());
			}
		}catch(Exception ex){
			ex.printStackTrace();
		}
		
		/*
		 * Getting historic trades/orders example 
		 */
		try{
			String startDate = DateUtil.getStartDate();
			String endDate = DateUtil.getEndDate();
			
			System.out.println("\n\nGetting yesterday's historic trades/orders (between "+startDate+" and " +endDate+")");

			ClosedTradesResponseMessage historyTradesOrders = tradingGateway.getHistory(startDate, endDate);
			if(historyTradesOrders.isSuccess()){
				System.out.println("Closed trades: ");
				System.out.println("Total: " +historyTradesOrders.getClosedPositions().length);
				for(TradeClosedResponseMessage closedTrade : historyTradesOrders.getClosedPositions()){
					System.out.print("providerTicket: " +closedTrade.getUniqueId());
					System.out.print(", brokerTicket: " + closedTrade.getBrokerTicket());
					System.out.print(", currency: " + closedTrade.getCurrencyName());
					System.out.print(", lots: " +closedTrade.getLots());
					System.out.print(", buy: " +closedTrade.isBuy());
					System.out.print(", PriceOpened: " +closedTrade.getPriceOpen());
					System.out.print(", PriceClosed: " +closedTrade.getPriceClosed());
					System.out.print(", DateOpened: " +closedTrade.getDateOpen());
					System.out.print(", openUtcTimeStamp: " +closedTrade.getOpenUtcTimestamp());
					System.out.print(", DateClosed: " +closedTrade.getDateClosed());
					System.out.print(", closeUtcTimeStamp: " +closedTrade.getCloseUtcTimestamp());
					System.out.print(", GrossPnl: " +closedTrade.getGrossPnl());
					System.out.print(", Commission: " +closedTrade.getCommission());
					System.out.print(", Interest: " +closedTrade.getInterest());
					System.out.print(", NetPnl: " +closedTrade.getNetPnl());
					System.out.println();
				}
				System.out.println("\nCanceled pending orders: ");
				System.out.println("Total: " +historyTradesOrders.getCanceledOrders().length);
				for(OrderCanceledResponseMessage canceledOrder : historyTradesOrders.getCanceledOrders()){
					System.out.print("providerTicket: " +canceledOrder.getUniqueId());
					System.out.print(", brokerTicket: " +canceledOrder.getBrokerTicket());
					System.out.print(", currency: " +canceledOrder.getCurrencyName());
					System.out.print(", lots: " +canceledOrder.getLots());
					System.out.print(", buy: " +canceledOrder.isBuy());
					System.out.print(", PriceOpened: "+canceledOrder.getPriceOpen());
					System.out.print(", DateOpened: " +canceledOrder.getDateOpened());
					System.out.print(", openedUtcTimestamp: " +canceledOrder.getOpenedUtcTimestamp());
					System.out.print(", DateCanceled: " +canceledOrder.getDateCanceled());
					System.out.print(", canceledUtcTimestamp: " +canceledOrder.getCanceledUtcTimestamp());
					System.out.println();
				}
				
			}else{
				System.err.println("\tCould not get historic trades/orders, response: "+historyTradesOrders.getErrorMessage()+", error code: "+historyTradesOrders.getErrorCode());
			}
			
		}catch(Exception ex){
			ex.printStackTrace();
		}

		
		/*
		 * Getting statistics of the account, if account is provider
		 */
		try{
			ProviderStatisticsResponseMessage providerStatisticsResponse = tradingGateway.getProviderStatistics();
			
			System.out.println("\n\nGetting provider's statistics: ");
			if(providerStatisticsResponse.getSuccess()){
				System.out.println("Position in rank: " +providerStatisticsResponse.getZuluRank());
				System.out.println("Amount Following: $" +providerStatisticsResponse.getAmountFollowing());
				System.out.println("Profit: " +providerStatisticsResponse.getProfit()+ " pips");
				System.out.println("Weeks: " +providerStatisticsResponse.getWeeks());
				System.out.println("Followers: " +providerStatisticsResponse.getFollowers());
				System.out.println("Trades: " +providerStatisticsResponse.getTrades());
				System.out.println("Winning trades: " +providerStatisticsResponse.getTotalWinTrades()+ " ("+providerStatisticsResponse.getTotalWinTradesPercent()+"%)");
				System.out.println("Average pips/trade: " +providerStatisticsResponse.getAvgPipsPerTrade());
				System.out.println("Average trade time: " +providerStatisticsResponse.getAvgTradeTimeInSecs()+ " seconds");
				System.out.println("Maximum Drawdown: " +providerStatisticsResponse.getMaxDrawDownPercent()+ "% ("+providerStatisticsResponse.getMaxDrawDown()+")");
				System.out.println("Max Open Trades: " +providerStatisticsResponse.getMaxOpenTrades());
				System.out.println("Worst Trade: " +providerStatisticsResponse.getWorstTradePips()+" pips");
				System.out.println("Best Trade: " +providerStatisticsResponse.getBestTradePips()+" pips");
				System.out.println("Has Live Followers: " +providerStatisticsResponse.getHasLiveFollowers());
				System.out.println("Necessary Minimum Equity: $" +providerStatisticsResponse.getNecessaryMinEquity());
				System.out.println("Statistics last updated in utc: " +providerStatisticsResponse.getStatsLastUpdateDate()+ ", timestamp: " +providerStatisticsResponse.getStatsLastUpdateDateUtcTimeStamp());
				System.out.println();
			}else{
				System.err.println("\tFailure: could not get provider statistics, error message: "+providerStatisticsResponse.getErrorMessage()+", error code: "+providerStatisticsResponse.getErrorCode());
			}
			
		}catch(Exception ex){
			ex.printStackTrace();
		}
	}
}
